@@ -17,7 +17,7 @@
 #along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 r"""
-Parsing and validation of URIs (RFC 3896) and IRIs (RFC 3987).
+Parsing and validation of URIs (RFC 3986) and IRIs (RFC 3987).
 
 This module provides regular expressions according to `RFC 3986 "Uniform 
 Resource Identifier (URI): Generic Syntax"
@@ -25,37 +25,55 @@ Resource Identifier (URI): Generic Syntax"
 Resource Identifiers (IRIs)" <http://tools.ietf.org/html/rfc3987>`_, and
 utilities for composition and relative resolution of references.
 
-Tested on python 2.7 and 3.2. Some features require regex_.
+Tested on python 2.7, 3.2 and 3.3. Some features require regex_.
+
+Note for python<=3.2: characters beyond the Basic Multilingual Plane are not
+supported on narrow builds (see `issue12729
+<http://bugs.python.org/issue12729>`_).
 
 
 API
 ---
 
-`get_compiled_pattern`
-    {get_compiled_pattern.__doc__}
+**match** (string, rule='IRI_reference')
+    {match.__doc__}
 
-`parse`
+**parse** (string, rule='IRI_reference')
     {parse.__doc__}
 
-`format_patterns`
-    {format_patterns.__doc__}
-    
-`patterns`
+**compose** (\*\*parts)
+    {compose.__doc__}
+
+**resolve** (base, uriref, strict=True, return_parts=False)
+    {resolve.__doc__}
+
+**patterns**
     A dict of regular expressions with useful group names.
     Compilable (with regex_ only) without need for any particular compilation
     flag.
 
-`compose`
-    {compose.__doc__}
+**[bmp_][u]patterns[_no_names]**
+    Alternative versions of `patterns`.
+    [u]nicode strings without group names for the re_ module.
+    BMP only for narrow builds.
 
-`resolve`
-    {resolve.__doc__}
+**get_compiled_pattern** (rule, flags=0)
+    {get_compiled_pattern.__doc__}
 
+**format_patterns** (\*\*names)
+    {format_patterns.__doc__}
 
 
 What's new
 ----------
 
+version 1.3.2:
+
+- convenience function `match`
+- patterns restricted to the BMP for narrow builds
+- adapted doctests for python 3.3
+- compatibility with python 2.6 (thanks to Thijs Janssen)
+
 version 1.3.1:
 
 - some re_ compatibility: get_compiled_pattern, parse
@@ -75,7 +93,11 @@ version 1.2.1:
 .. _regex: http://pypi.python.org/pypi/regex
 
 """
-__version__ = '1.3.1'
+__version__ = '1.3.2'
+
+import sys
+
+NARROW_BUILD = sys.maxunicode == 0xffff
 
 try:
     basestring
@@ -89,8 +111,8 @@ except ImportError:
     import re as _re
     REGEX = False
 
-__all__ = ('patterns', 'format_patterns', 'get_compiled_pattern',
-           'compose', 'resolve')
+__all__ = ('get_compiled_pattern', 'parse', 'format_patterns', 'patterns',
+           'compose', 'resolve', 'match')
 
 
 _common_rules = (
@@ -238,42 +260,24 @@ def format_patterns(**names):
     r"""Returns a dict of patterns (regular expressions) keyed by
     `rule names for URIs`_ and `rule names for IRIs`_.
     
-    See also the module level dict `patterns`, and `get_compiled_pattern`.
+    See also the module level dicts of patterns, and `get_compiled_pattern`.
 
-    To wrap a rule in a named capture group, pass as it keyword argument:
+    To wrap a rule in a named capture group, pass it as keyword argument:
     rule_name='group_name'. By default, the formatted patterns contain no
     named groups.
 
     Patterns are `str` instances (be it in python 2.x or 3.x) containing ASCII
     characters only.
 
-    Note that, *if* compiling with the standard library re_ module:
+    Caveats::
 
-      - ``\u`` and ``\U`` escapes must be preprocessed (see `issue3665
-        <http://bugs.python.org/issue3665>`_)::
-        
-          >>> import re, sys, ast
-          >>> re.compile(patterns['ucschar']) #doctest:+IGNORE_EXCEPTION_DETAIL
-          Traceback (most recent call last):
-            ...
-            File "/usr/lib/python2.6/re.py", line 245, in _compile
-              raise error, v # invalid expression
-          error: bad character range
-          >>> tpl = 'u"%s"' if sys.version_info[0] < 3 else '"%s"'
-          >>> utext_pattern = ast.literal_eval(tpl % patterns['ucschar'])
-          >>> assert re.compile(utext_pattern)
+      - with re_, named capture groups cannot occur on multiple branches of an
+        alternation
 
-      - named capture groups cannot occur on multiple branches of an
-        alternation::
+      - with re_ before python 3.3, ``\u`` and ``\U`` escapes must be
+        preprocessed (see `issue3665 <http://bugs.python.org/issue3665>`_)::
 
-          >>> re.compile(patterns['path']) #doctest:+IGNORE_EXCEPTION_DETAIL
-          Traceback (most recent call last):
-            ...
-            File "/usr/lib/python2.6/re.py", line 245, in _compile
-              raise error, v # invalid expression
-          error: redefinition of group name 'path' as group 2; was group 1
-          >>> pat = format_patterns(path='outermost_group_name')['path']
-          >>> assert re.compile(pat)
+      - on narrow builds, character ranges beyond BMP are not supported
 
     .. _rule names for URIs: http://tools.ietf.org/html/rfc3986#appendix-A
     .. _rule names for IRIs: http://tools.ietf.org/html/rfc3987#section-2.2
@@ -290,7 +294,7 @@ def format_patterns(**names):
     return formatted
 
 
-_GROUP_NAMES_BASE = [2*[n] for n in [
+_GROUP_NAMES_BASE = [
         'scheme', 'port',
         'IPv6address', 'IPv4address', 'IPvFuture',
         'URI_reference',
@@ -301,9 +305,9 @@ _GROUP_NAMES_BASE = [2*[n] for n in [
         'IRI', 'absolute_IRI', 'irelative_ref', 'irelative_part',
         'iauthority', 'ihost', 'iuserinfo', 'ireg_name',
         'iquery', 'ifragment'
-        ]]
+        ]
 
-DEFAULT_GROUP_NAMES = dict(_GROUP_NAMES_BASE,
+DEFAULT_GROUP_NAMES = dict(zip(_GROUP_NAMES_BASE,_GROUP_NAMES_BASE),
     path_abempty='path', path_absolute='path', path_noscheme='path',
     path_rootless='path', path_empty='path',
     ipath_abempty='ipath', ipath_absolute='ipath', ipath_noscheme='ipath',
@@ -314,14 +318,24 @@ patterns = format_patterns(**DEFAULT_GROUP_NAMES)
 
 
 def _interpret_unicode_escapes(string):
-    import sys, ast
-    tpl = 'u"""{}"""' if sys.version_info[0] < 3 else '"""{}"""'
+    import ast
+    tpl = 'u"""{0}"""' if sys.version_info[0] < 3 else '"""{}"""'
     return ast.literal_eval(tpl.format(string))
 
-if not REGEX:
-    #: compilable with re
-    _upatterns_no_names = dict((k, _interpret_unicode_escapes(v)) for k,v
-                              in format_patterns().items())
+patterns_no_names = format_patterns()
+
+# if not REGEX:
+#: patterns compilable with re
+upatterns_no_names = dict((k, _interpret_unicode_escapes(v)) for k,v
+                          in format_patterns().items())
+
+_bmp = lambda s: _re.sub(r'\\U[0-9A-F]{8}-\\U[0-9A-F]{8}', '', s)
+#: patterns restricted to the basic multilingual plane
+#: compilable on narrow build
+bmp_patterns = dict((k, _bmp(v)) for k,v in patterns.items())
+#: compilable on narrow build with re
+bmp_upatterns_no_names = dict((k, _interpret_unicode_escapes(_bmp(v)))
+                              for k,v in patterns_no_names.items())
 
 
 def get_compiled_pattern(rule, flags=0):
@@ -331,12 +345,16 @@ def get_compiled_pattern(rule, flags=0):
         
         >>> uri = get_compiled_pattern('^%(URI)s$')
         >>> assert uri.match('http://tools.ietf.org/html/rfc3986#appendix-A')
+        >>> assert not get_compiled_pattern('^%(relative_ref)s$').match('#f#g')
         >>> from unicodedata import lookup
         >>> smp = 'urn:' + lookup('OLD ITALIC LETTER A')  # U+00010300
         >>> assert not uri.match(smp)
-        >>> assert get_compiled_pattern('^%(IRI)s$').match(smp)
-        >>> assert not get_compiled_pattern('^%(relative_ref)s$').match('#f#g')
-    
+        >>> m = get_compiled_pattern('^%(IRI)s$').match(smp)
+
+    On narrow builds, non-BMP characters are (incorreclty) excluded::
+
+        >>> assert NARROW_BUILD == (not m)
+
     For parsing, some subcomponents are captured in named groups (*only if*
     regex_ is available, otherwise see `parse`)::
 
@@ -349,16 +367,27 @@ def get_compiled_pattern(rule, flags=0):
         ...                  d['query'] == None,
         ...                  d['fragment'] == 'appendix-A' ])
 
+        >>> for r in patterns.keys():
+        ...     assert get_compiled_pattern(r)
+
     """
     cache, key = get_compiled_pattern.cache, (rule, flags)
     if key not in cache:
-        pats = patterns if REGEX else _upatterns_no_names
-        p = pats.get(rule) or rule % pats  #.format(**pats)
+        if NARROW_BUILD:
+            pats = bmp_patterns if REGEX else bmp_upatterns_no_names
+        else:
+            pats = patterns if REGEX else upatterns_no_names
+        p = pats.get(rule) or rule % pats
         cache[key] = _re.compile(p, flags)
     return cache[key]
 get_compiled_pattern.cache = {}
 
 
+def match(string, rule='IRI_reference'):
+    """Returns a match object or None."""
+    return get_compiled_pattern('^%%(%s)s$' % rule).match(string)
+
+
 #: http://tools.ietf.org/html/rfc3986#appendix-B
 _iri_non_validating_re = _re.compile(
     r"^((?P<scheme>[^:/?#]+):)?(//(?P<authority>[^/?#]*))?"
@@ -372,6 +401,9 @@ REFERENCE_RULES = ('IRI_reference', 'IRI', 'absolute_IRI',
 def parse(string, rule='IRI_reference'):
     """Parses `string` according to `rule` into a dict of subcomponents.
 
+    If `rule` is None, parse an IRI_reference `without validation
+    <http://tools.ietf.org/html/rfc3986#appendix-B>`_.
+
     If regex_ is available, any rule is supported; with re_, `rule` must be
     'IRI_reference' or some special case thereof ('IRI', 'absolute_IRI',
     'irelative_ref', 'irelative_part', 'URI_reference', 'URI', 'absolute_URI',
@@ -384,12 +416,12 @@ def parse(string, rule='IRI_reference'):
         ...              d['path'] == '/html/rfc3986',
         ...              d['query'] == None,
         ...              d['fragment'] == 'appendix-A' ])
-        
+
     """
-    if not REGEX and rule not in REFERENCE_RULES:
+    if not REGEX and rule and rule not in REFERENCE_RULES:
         raise ValueError(rule)
     if rule:
-        m = get_compiled_pattern('^%%(%s)s$' % rule).match(string)
+        m = match(string, rule)
         if not m:
             raise ValueError('%r is not a valid %r.' % (string, rule))
         if REGEX:
